import abjad

import nauert


def test_QGrid_distance_01():
    q_grid = nauert.QGrid()
    assert q_grid.distance is None

    a = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(0), ["A"]),
        abjad.duration.offset(0),
    )
    q_grid.fit_q_events([a])
    assert q_grid.distance == abjad.duration.offset(0).duration()

    b = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(1, 20), ["B"]),
        abjad.duration.offset(1, 20),
    )
    q_grid.fit_q_events([b])
    assert q_grid.distance == abjad.duration.offset(1, 40).duration()

    c = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(9, 20), ["C"]),
        abjad.duration.offset(9, 20),
    )
    q_grid.fit_q_events([c])
    assert q_grid.distance == abjad.duration.offset(1, 6).duration()

    d = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(1, 2), ["D"]),
        abjad.duration.offset(1, 2),
    )
    q_grid.fit_q_events([d])
    assert q_grid.distance == abjad.duration.offset(1, 4).duration()

    e = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(11, 20), ["E"]),
        abjad.duration.offset(11, 20),
    )
    q_grid.fit_q_events([e])
    assert q_grid.distance == abjad.duration.offset(29, 100).duration()

    f = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(19, 20), ["F"]),
        abjad.duration.offset(19, 20),
    )
    q_grid.fit_q_events([f])
    assert q_grid.distance == abjad.duration.offset(1, 4).duration()

    g = nauert.QEventProxy(
        nauert.SilentQEvent(abjad.duration.offset(1), ["G"]),
        abjad.duration.offset(1),
    )
    q_grid.fit_q_events([g])
    assert q_grid.distance == abjad.duration.offset(3, 14).duration()

    q_events = q_grid.subdivide_leaves([(0, (1, 1))])
    q_grid.fit_q_events(q_events)

    assert q_grid.distance == abjad.duration.offset(1, 35).duration()



import abjad

import nauert


def test_QuantizationJob___call___01():
    job_id = 1
    definition = {2: {2: {2: None}, 3: None}, 5: None}
    search_tree = nauert.UnweightedSearchTree(definition)
    q_event_proxies = [
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(0), ["A"], index=1),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(1, 5), ["B"], index=2),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(1, 4), ["C"], index=3),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(1, 3), ["D"], index=4),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(2, 5), ["E"], index=5),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(1, 2), ["F"], index=6),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(3, 5), ["G"], index=7),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(2, 3), ["H"], index=8),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(3, 4), ["I"], index=9),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(4, 5), ["J"], index=10),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
        nauert.QEventProxy(
            nauert.SilentQEvent(abjad.duration.offset(1), ["K"], index=11),
            abjad.duration.offset(0),
            abjad.duration.offset(1),
        ),
    ]
    job = nauert.QuantizationJob(job_id, search_tree, q_event_proxies)
    job()
    assert len(job.q_grids) == 10

    rtm_formats = [q_grid.root_node.rtm_format() for q_grid in job.q_grids]
    rtm_formats.sort(reverse=True)
    assert rtm_formats == [
        "1",
        "(1 (1 1))",
        "(1 (1 1 1 1 1))",
        "(1 ((1 (1 1)) (1 (1 1))))",
        "(1 ((1 (1 1)) (1 (1 1 1))))",
        "(1 ((1 (1 1 1)) (1 (1 1))))",
        "(1 ((1 (1 1 1)) (1 (1 1 1))))",
        "(1 ((1 (1 1 1)) (1 ((1 (1 1)) (1 (1 1))))))",
        "(1 ((1 ((1 (1 1)) (1 (1 1)))) (1 (1 1 1))))",
        "(1 ((1 ((1 (1 1)) (1 (1 1)))) (1 ((1 (1 1)) (1 (1 1))))))",
    ], rtm_formats



import abjad

import nauert


def test_QEventSequence_from_millisecond_pitch_attachment_tuples_01():
    durations = [100, 200, 100, 300, 350, 400, 600]
    pitches = [0, None, None, [1, 4], None, 5, 7]
    attachments = [("foo",), (), (), (6,), (), ("foobar",), ("foo", "bar")]
    tuples = tuple(zip(durations, pitches, attachments, strict=True))
    q_events = nauert.QEventSequence.from_millisecond_pitch_attachment_tuples(tuples)
    assert q_events == nauert.QEventSequence(
        (
            nauert.PitchedQEvent(
                abjad.duration.offset(0), (abjad.NamedPitch("c'"),), ("foo",)
            ),
            nauert.SilentQEvent(abjad.duration.offset(100, 1)),
            nauert.PitchedQEvent(
                abjad.duration.offset(400, 1),
                (abjad.NamedPitch("cs'"), abjad.NamedPitch("e'")),
                (6,),
            ),
            nauert.SilentQEvent(abjad.duration.offset(700, 1)),
            nauert.PitchedQEvent(
                abjad.duration.offset(1050, 1),
                (abjad.NamedPitch("f'"),),
                ("foobar",),
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(1450, 1),
                (abjad.NamedPitch("g'"),),
                ("foo", "bar"),
            ),
            nauert.TerminalQEvent(abjad.duration.offset(2050, 1)),
        )
    ), repr(q_events)



import abjad

import nauert


def test_QEventSequence_from_tempo_scaled_leaves_01():
    staff = abjad.Staff("c'4 r4 r8 cs'8 cs'8 d'8 d'8 ef'8 s4 r4 ef'8")
    staff.append("<c' cs' e'>4")
    abjad.tie(staff[3:5])
    abjad.tie(staff[5:7])
    tempo = abjad.MetronomeMark(abjad.Duration(1, 4), 55)
    leaves = abjad.select.leaves(staff)
    q_events = nauert.QEventSequence.from_tempo_scaled_leaves(leaves, tempo)
    assert q_events == nauert.QEventSequence(
        (
            nauert.PitchedQEvent(
                abjad.duration.offset(0, 1), (abjad.NamedPitch("c'"),)
            ),
            nauert.SilentQEvent(abjad.duration.offset(12000, 11)),
            nauert.PitchedQEvent(
                abjad.duration.offset(30000, 11), (abjad.NamedPitch("cs'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(42000, 11), (abjad.NamedPitch("d'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(54000, 11), (abjad.NamedPitch("ef'"),)
            ),
            nauert.SilentQEvent(abjad.duration.offset(60000, 11)),
            nauert.PitchedQEvent(
                abjad.duration.offset(84000, 11), (abjad.NamedPitch("ef'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(90000, 11),
                (
                    abjad.NamedPitch("c'"),
                    abjad.NamedPitch("cs'"),
                    abjad.NamedPitch("e'"),
                ),
            ),
            nauert.TerminalQEvent(abjad.duration.offset(102000, 11)),
        )
    )


def test_QEventSequence_from_tempo_scaled_leaves_02():
    staff = abjad.Staff("c'4 r4 r8 cs'8 cs'8 d'8 d'8 ef'8 s4 r4 ef'8")
    staff.append("<c' cs' e'>4")
    abjad.tie(staff[3:5])
    abjad.tie(staff[5:7])
    tempo = abjad.MetronomeMark(abjad.Duration(1, 4), 58)
    abjad.attach(tempo, staff[0], context="Staff")
    tempo = abjad.MetronomeMark(abjad.Duration(1, 4), 77)
    abjad.attach(tempo, staff[9], context="Staff")
    leaves = abjad.select.leaves(staff)
    q_events = nauert.QEventSequence.from_tempo_scaled_leaves(leaves)
    assert q_events == nauert.QEventSequence(
        (
            nauert.PitchedQEvent(
                abjad.duration.offset(0, 1), (abjad.NamedPitch("c'"),)
            ),
            nauert.SilentQEvent(abjad.duration.offset(30000, 29)),
            nauert.PitchedQEvent(
                abjad.duration.offset(75000, 29), (abjad.NamedPitch("cs'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(105000, 29), (abjad.NamedPitch("d'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(135000, 29), (abjad.NamedPitch("ef'"),)
            ),
            nauert.SilentQEvent(abjad.duration.offset(150000, 29)),
            nauert.PitchedQEvent(
                abjad.duration.offset(15600000, 2233), (abjad.NamedPitch("ef'"),)
            ),
            nauert.PitchedQEvent(
                abjad.duration.offset(16470000, 2233),
                (
                    abjad.NamedPitch("c'"),
                    abjad.NamedPitch("cs'"),
                    abjad.NamedPitch("e'"),
                ),
            ),
            nauert.TerminalQEvent(abjad.duration.offset(18210000, 2233)),
        )
    )
